import 'CoreLibs/animator.lua'
import 'CoreLibs/easing.lua'
import 'CoreLibs/graphics.lua'
import 'CoreLibs/object.lua'
import 'CoreLibs/ui.lua'

local gfx = playdate.graphics
playdate.display.setRefreshRate(50)
gfx.setBackgroundColor(gfx.kColorWhite)

local grassBackground = gfx.image.new("images/game/grass.png")

local pitchFlyTimeMs = 2500
local ballStartY, endY = 90, 250
local easingFunction = playdate.easingFunctions.outQuint
local pitchAnimator = gfx.animator.new(pitchFlyTimeMs, ballStartY, endY, easingFunction)

local ballSizeMs = 2000
local ballSizeAnimator = gfx.animator.new(ballSizeMs, 9, 6, playdate.easingFunctions.outBounce)

local screenW, screenH = 400, 240
local centerX, centerY = screenW / 2, screenH / 2
local batBaseX, batBaseY = centerX - 34, 220

local batLength = 45

local ballY = ballStartY
local lastBallY = ballY
local ballX = 200
local ballVelX, ballVelY = 0, 0
local ballSize = 6

local batTipX = 0
local batTipY = 0

local hit = false

local hitAnimatorY
local hitAnimatorX

local hitMult = 10

function hitBall(destX, destY, hitFlyTime)
    ballSizeAnimator:reset()
    hitAnimatorY = gfx.animator.new(hitFlyTime, ballY, destY, playdate.easingFunctions.outQuint)
    hitAnimatorX = gfx.animator.new(hitFlyTime, ballX, destX, playdate.easingFunctions.outQuint)
end

function pitch()
    pitchAnimator:reset()
    ballVelX = 0
    ballVelY = 0
    ballX = 200
    hit = false
end

function playdate.AButtonDown()
    pitch()
end

function playdate.upButtonDown()
    batBaseY -= 1
end

function playdate.downButtonDown()
    batBaseY += 1
end

function playdate.rightButtonDown()
    batBaseX += 1
end

function playdate.leftButtonDown()
    batBaseX -= 1
end

local pitchClockSec = 5
local elapsedTime = 0

function ballPassedThruBat(ballX, ballY, batBaseX, batBaseY, batTipX, batTipY)
    -- This check currently assumes right-handedness.
    -- I.e. it assumes the ball is to the right of batBaseX
    if ballX < batBaseX or ballX > batTipX or ballY > screenH then
        return false
    end

    local m = (batTipY - batBaseY) / (batTipX - batBaseX)

    -- y = mx + b
    -- b = y1 - (m * x1)
    local b = batBaseY - (m * batBaseX)
    local yOnLine = (m * ballX) + b
    local yP = ballY
    local yDelta = yOnLine - yP

    return yDelta <= 0
end

function updateGameState()
    local deltaTime = playdate.getElapsedTime()
    playdate.resetElapsedTime()
    elapsedTime = elapsedTime + deltaTime

    if elapsedTime > pitchClockSec then
        elapsedTime = 0
        pitch()
    end

    if hit then
        ballX = hitAnimatorX:currentValue()
        ballY = hitAnimatorY:currentValue()
        ballSize = ballSizeAnimator:currentValue()
    else
        ballY = pitchAnimator:currentValue()
        ballSize = 6
    end

    batAngle = math.rad(playdate.getCrankPosition() + 90)
    batTipX = batBaseX + (batLength * math.sin(batAngle))
    batTipY = batBaseY + (batLength * math.cos(batAngle))

    if hit == false and ballPassedThruBat(ballX, ballY, batBaseX, batBaseY, batTipX, batTipY) then
        ballAngle = batAngle + math.rad(90)

        crankChange, acceleratedChange = playdate.getCrankChange()
        mult = math.abs(acceleratedChange / 15)
        ballVelX = mult * 10 * math.sin(ballAngle)
        ballVelY = mult * 5 * math.cos(ballAngle)
        if ballVelY > 0 then
            ballVelX = ballVelX * -1
            ballVelY = ballVelY * -1
        end
        ballDestX = ballX + (ballVelX * hitMult)
        ballDestY = ballY + (ballVelY * hitMult)
        hitBall(ballDestX, ballDestY, 2000)
        hit = true
    end
    lastBallY = ballY
end

function playdate.update()
    updateGameState()

    grassBackground:draw(0, 0)

    gfx.setColor(gfx.kColorBlack)
    gfx.setLineWidth(2)
    gfx.drawCircleAtPoint(ballX, ballY, ballSize)

    gfx.setLineWidth(5)
    gfx.drawLine(batBaseX, batBaseY, batTipX, batTipY)

    if playdate.isCrankDocked() then
        playdate.ui.crankIndicator:draw()
    end
end
